@@ -1,4 +1,6 @@
 import json, sys
+import traceback
+
 from PyQt5 import QtWidgets, QtCore
 from PyQt5.QtGui import *
 from PyQt5.QtCore import *
@@ -95,7 +97,8 @@ class PyHtmlQtSimpleTray(GenericTray):
 class PyHtmlQtTray(GenericTray):
     def __init__(self, pyHtmlQtApp, url, size, icon_path):
         super().__init__(pyHtmlQtApp, icon_path)
-        self.webWidget = PyHtmlWebWidget(url, size)
+        self.webWidget = PyHtmlWebWidget(url, size=size)
+        self.webWidget.load_page()
         self.trayAction = QWidgetAction(self.tray)
         self.trayAction.setDefaultWidget(self.webWidget)
         self.menu.addAction(self.trayAction)
@@ -106,9 +109,11 @@ class PyHtmlQtTray(GenericTray):
     def removeJavascriptFunction(self, name):
         self.webWidget.removeJavascriptFunction(name)
 
+    def runJavascript(self, javascript, callback=None):
+        self.webWidget.runJavascript(javascript, callback)
 
 class PyHtmlQtWindow():
-    def __init__(self, pyHtmlQtApp, url, size, title, icon_path=None):
+    def __init__(self, pyHtmlQtApp, url, size, title, icon_path=None, keep_connected_on_close = False, keep_connected_on_minimize = True):
         self.pyHtmlQtApp = pyHtmlQtApp
         self.url = url
         self.size = size
@@ -119,19 +124,23 @@ class PyHtmlQtWindow():
         self.on_show_event = Observable()
         self.on_minimized_event = Observable()
 
-        #self.setFocusPolicy(Qt.StrongFocus)
-
-        self._webWidget = PyHtmlWebWidget(url, size)
+        self._webWidget = PyHtmlWebWidget(url)
         self._qMainWindow = ExtendedQMainWindow(self) # you could also let this class directly subclass QMainWindow, but then PyHtmlQtWindow will expose a s*load of confusing qt functions from QMainWindow
+        self._qMainWindow.resize(size[0], size[1])
+        self._qMainWindow.setFocusPolicy(Qt.StrongFocus)
         self._qMainWindow.setWindowTitle(self.title)
         self._qMainWindow.setCentralWidget(self._webWidget)
-        self._qMainWindow.setAttribute(Qt.WA_QuitOnClose, False)
         self._webWidget.setParent(self._qMainWindow)
-
         self._menuBar = QMenuBar()
         self._menuBar._subitems = {} # qt also keep track of this, but this is easier for the addMenuButton and addMenuSeparator functions
         self._qMainWindow.setMenuBar(self._menuBar)
 
+        if keep_connected_on_close is False:
+            self.on_closed_event.attach_observer(self._webWidget.unload_page)
+        if keep_connected_on_minimize is False:
+            self.on_minimized_event.attach_observer(self._webWidget.unload_page)
+        self.on_show_event.attach_observer(self._webWidget.load_page)
+
     def show(self, *args):
         self._qMainWindow.showNormal()
         self._qMainWindow.activateWindow()
@@ -164,6 +173,9 @@ class PyHtmlQtWindow():
     def removeJavascriptFunction(self, name):
         self._webWidget.removeJavascriptFunction(name)
 
+    def runJavascript(self, javascript, callback=None):
+        self._webWidget.runJavascript(javascript, callback)
+
     def _get_submenu(self, names):
         parent = self._menuBar
         for name in names:
@@ -178,16 +190,19 @@ class PyHtmlQtWindow():
 
 
 class PyHtmlWebWidget(QWidget):
-    def __init__(self, url , size):
+    def __init__(self, url, size=None):
         super().__init__()
         self.url  = url
+        self._current_url = None
         self.size = size
 
         self.web = QWebEngineView(parent=self)
+        if self.size is not None:
+            self.web.setFixedSize(size[0], size[1])
+
         self.profile = QWebEngineProfile(parent= self) # create a seperate profile for earch webengine, otherwise QWebEngineViews will load only the first loaded url in all view.
         self.web.setPage(QWebEnginePage(self.profile, self.web))
-        self.web.setFixedSize(self.size[0], self.size[1])
-        self.web.load(QUrl(self.url))
+        self.web.load(QUrl("about:blanc"))
 
         self.channel = QWebChannel(parent=self.web)
         self._functionHandler = JavascriptFunctionHandler()
@@ -201,12 +216,28 @@ class PyHtmlWebWidget(QWidget):
         self.layout.setRowStretch(0, 0)
         self.setLayout(self.layout)
 
+    def load_page(self, *args):
+        if self._current_url != self.url:
+            self._current_url = self.url
+            self.web.load(QUrl(self.url))
+
+    def unload_page(self, *args):
+        if self._current_url != None:
+            self._current_url = None
+            self.web.load(QUrl("about:blanc"))
+
     def addJavascriptFunction(self, name, target):
         self._functionHandler.add(name, target)
 
     def removeJavascriptFunction(self, name):
         self._functionHandler.remove(name)
 
+    def runJavascript(self, javascript, callback=None):
+        if callback is None:
+            self.web.page().runJavaScript(javascript)
+        else:
+            self.web.page().runJavaScript(javascript, callback)
+
 
 class ExtendedQMainWindow(QMainWindow):
     def __init__(self, pyHtmlQtWindow):
@@ -230,7 +261,16 @@ class JavascriptFunctionHandler(QObject):
 
     @pyqtSlot(str, str, result=str)
     def call(self, action_name, arguments):
-        return json.dumps(self._actions[action_name](*json.loads(arguments)))
+        if action_name not in self._actions:
+            return json.dumps({"exception": "Unknown function pyhtmlapp.%s, did you forget to call addJavascriptFunction ?" % action_name})
+        try:
+            arguments = json.loads(arguments) # do in multiple lines, so if something failes, we see where.
+            action = self._actions[action_name]
+            result = action(*arguments)
+            return json.dumps({"result" : result})
+        except Exception as e:
+            msg = "Exception while calling python from javascript function 'pyhtmlapp.%s'\n%s" %  (action_name, traceback.format_exc())
+            return json.dumps({"exception": msg })
 
     def add(self, name, target):
         self._actions[name] = target
