@@ -2,17 +2,19 @@ from __future__ import annotations
 
 import json
 import os
+import random
 import shutil
 import sys
 import threading
 import time
 import traceback
 import uuid
+import webbrowser
+
 import jinja2
 import typing
 from werkzeug.serving import make_server
 
-from .lib import Browser
 from .pyhtmlguiInstance import PyHtmlGuiInstance
 from .view import PyHtmlView
 import flask, flask_sock
@@ -30,8 +32,6 @@ class PyHtmlGui:
                  on_view_disconnected: typing.Callable = None,
                  size             : tuple[int, int] = None,
                  position         : tuple[int, int] = None,
-                 browser          : str             = "default",
-                 executable       : str             = None,
                  listen_host      : str             = "127.0.0.1",
                  listen_port      : int             = 8000,
                  shared_secret    : str             = None,
@@ -40,6 +40,7 @@ class PyHtmlGui:
                  ) -> None:
         """
         :param app_instance: Some object (eg. main program class instance), passed to view_class as obj on launch
+        :type  app_instance: object
         :param view_class: A class that Inherits from PyHtmlView
         :param static_dir: Static files, css, img go here
         :param template_dir: main.html and other html goes here
@@ -52,8 +53,6 @@ class PyHtmlGui:
         :param on_view_disconnected: callback is called when a frontend websocket is disconnected
                                         arguments passed: "nr of view instances", "nr of websocket connections"
         :param position: window position
-        :param browser: default | chrome | electron
-        :param executable: path to chrome/electron executable, if needed
         :param listen_host:
         :param listen_port:
         :param shared_secret: use "" to automatically generate a uid internally, None to disable token
@@ -77,9 +76,6 @@ class PyHtmlGui:
         )
         self.static_dir = None if static_dir is None else os.path.abspath(static_dir)
         self.template_dir = None if template_dir is None else os.path.abspath(template_dir)
-        self.browser = browser
-        self._browser = None
-        self.executable = executable
         self.electron_app_dir = electron_app_dir
         if shared_secret == "":
             self.shared_secret = "%s" % uuid.uuid4()
@@ -91,7 +87,7 @@ class PyHtmlGui:
         self.listen_port = listen_port
         self.auto_reload = auto_reload
 
-        if getattr(sys, 'frozen', False) is True:  # check if we are bundled by pyinstaller
+        if getattr(sys, 'frozen', False) is True:
             # noinspection PyUnresolvedReferences,PyProtectedMember
             lib_dir = os.path.join(sys._MEIPASS, "pyhtmlgui")
             self.auto_reload = False
@@ -122,12 +118,14 @@ class PyHtmlGui:
         self._token_csrf = "%s" % uuid.uuid4()
 
         self.flaskApp = flask.Flask("pyhtmlgui", static_folder=None, template_folder=None)
+
         self.flaskWebsocket =  flask_sock.Sock()
         self.flaskApp.route("/")(self._main_html)
         self.flaskApp.route("/<endpoint>")(self._main_html)
         if self.static_dir is not None:
             self.flaskApp.route("/static/<path:path>")(self._static)
         self.flaskWebsocket.route("/ws")(self._websocket)
+        #self.flaskWebsocket.route("/ws/<endpoint>")(self._websocket)
         self.flaskWebsocket.init_app(self.flaskApp)
 
         self._server = WebsocketServerThread(self.flaskApp, self.listen_host, self.listen_port)
@@ -138,12 +136,11 @@ class PyHtmlGui:
             t.start()
 
     def start(self, show_frontend: bool = False, block: bool = True) -> None:
+        self._server.start()
         if show_frontend is True:
             self.show()
         if block is True:
-            self._server.run()
-        else:
-            self._server.start()
+            self.join()
 
     def stop(self) -> None:
         # noinspection PyBroadException
@@ -152,25 +149,12 @@ class PyHtmlGui:
         except Exception:
             pass
 
-    def show(self) -> None:
-        env = None
-        target_host = "127.0.0.1" if self.listen_host == "0.0.0.0" else self.listen_host
+    def join(self):
+        self._server.join()
 
-        if self.browser == "electron":
-            args = [self.electron_app_dir, ]
-            env = os.environ.copy()
-            env.update({
-                "PYHTMLGUI_HOST": target_host,
-                "PYHTMLGUI_PORT": "%s" % self.listen_port,
-                "PYHTMLGUI_SECRET": self.shared_secret,
-            })
-        else:
-            args = ["%s:%s" % (target_host, self.listen_port), ]
-            if self.shared_secret is not None:
-                args[0] = "%s?token=%s" % (args[0], self.shared_secret)
-        if self._browser is None:
-            self._browser = Browser(self.browser, self.executable)
-        self._browser.open(args, env=env)
+    def show(self) -> None:
+        for endpoint in self._endpoints.values():
+            endpoint.show()
 
     def add_endpoint(self, app_instance: object, view_class: typing.Type[PyHtmlView], name: str = "", base_template: str = "pyHtmlGuiBase.html",
          on_view_connected: typing.Callable = None, on_view_disconnected: typing.Callable = None,
@@ -180,6 +164,7 @@ class PyHtmlGui:
         self._endpoints[name] = PyHtmlGuiEndpoint(
             parent        = self,
             app_instance  = app_instance,
+            name          = name,
             view_class    = view_class,
             base_template = base_template,
             on_view_connected    = on_view_connected,
@@ -189,6 +174,11 @@ class PyHtmlGui:
             position = position
         )
 
+    def get_url(self, endpoint = ""):
+        if endpoint in self._endpoints:
+            return self._endpoints[endpoint].get_url()
+        return None
+
     # /
     def _main_html(self, endpoint = ""):
         if self.shared_secret is not None and flask.request.args.get('token') != self.shared_secret:
@@ -235,7 +225,7 @@ class PyHtmlGui:
         return response
 
     # /ws
-    def _websocket(self, ws):
+    def _websocket(self, ws, endpoint= ""):
         endpoint = flask.request.cookies.get('endpoint',"")
         if flask.request.cookies.get('token') != self._token_cookie or flask.request.args.get('token') != self._token_csrf:
             msg = "token_failed"
@@ -249,7 +239,7 @@ class PyHtmlGui:
         except:
             pass
 
-    def add_file_to_monitor(self, file_to_monitor, class_name) -> None:
+    def _add_file_to_monitor(self, file_to_monitor, class_name) -> None:
         if self.auto_reload is False:
             return
         if file_to_monitor not in self._file_monitoring:
@@ -285,9 +275,8 @@ class PyHtmlGui:
 
 
 class WebsocketServerThread(threading.Thread):
-
     def __init__(self, app, host, port ):
-        threading.Thread.__init__(self)
+        threading.Thread.__init__(self, daemon=True)
         self.server = make_server(host, port, app, threaded=True)
         self.ctx = app.app_context()
         self.ctx.push()
@@ -337,3 +326,12 @@ class PyHtmlGuiEndpoint:
         if self.on_view_disconnected_callback is not None:
             self.on_view_disconnected_callback(len(self._gui_instances), sum([i.connections_count for i in self._gui_instances]))
 
+    def get_url(self):
+        target_host = "127.0.0.1" if self.parent.listen_host == "0.0.0.0" else self.parent.listen_host
+        url = "http://%s:%s/%s" % (target_host, self.parent.listen_port, self.name)
+        if self.parent.shared_secret is not None:
+            url = "%s?token=%s" % (url, self.parent.shared_secret)
+        return url
+
+    def show(self) -> None:
+        webbrowser.open(self.get_url(), 1)
